#!/bin/bash
#
# Reproducer for https://bugzilla.redhat.com/show_bug.cgi?id=2149406
# Bug 2149406 - umount() fails to unmount an NFS mounted autofs path where the exported filesystem is no longer exported to the client and the server returns NFS4ERR_PERM in response to GETATTR during unmount
#
# NOTE: This reproduces the error on both 'ls' and 'umount' when the exports are set unavailable.
# However, it does not reproduce the NFS server returning NFS4ERR_PERM in response to GETATTR.
# With the Linux NFS server the response is an AUTH_ERROR on various NFS / RPCs.
#
function check_service {
	systemctl status $1 >/dev/null 2>&1
	if [ $? -ne 0 ]; then
		echo Trying to start $1
		systemctl start $1 >/dev/null 2>&1
        fi
        if [ $? -ne 0 ]; then
                echo could not start $1
                exit 1;
        fi
}
function set_exports_available {
	test -f /etc/exports.bak || cp /etc/exports /etc/exports.bak
	mkdir -p /exports/dir1
	echo "/exports			127.0.0.1(rw,fsid=0)" > /etc/exports
	echo "/exports/dir1		127.0.0.1(rw,fsid=10)" >> /etc/exports
	exportfs -avu ; exportfs -av
}
function set_exports_unavailable {
	echo "/exports			1.2.3.4(rw,fsid=0,nocrossmnt)" 	> /etc/exports
	echo "/exports/dir1		1.2.3.4(rw,fsid=10)"	>> /etc/exports
	exportfs -avu ; exportfs -av
}

check_service nfs-server

echo setting exports available
set_exports_available
mount -overs=4.1 127.0.0.1:/ /mnt/exports
mount -overs=4.1 127.0.0.1:/dir1 /mnt/exports/dir1

echo setting exports unavailable
set_exports_unavailable

echo sleeping 60s to let attribute cache expire
sleep 60

# TODO: Check for error code / message
# ls: cannot access '/mnt/exports/dir1/dir2': Permission denied
ls -al /mnt/exports/dir1

# TODO: Check for error code / message?
# umount.nfs: /mnt/exports/dir1/dir2: block devices not permitted on fs
umount /mnt/exports/dir1
if [ $? -ne 0 ]; then
	echo TEST FAIL on $(uname -r)
	echo "output of 'grep 127.0.0.1 /proc/mounts'"
	grep 127.0.0.1 /proc/mounts
	set_exports_available
	exit 1
fi
echo TEST PASS on $(uname -r)
